@@ -1,38 +1,46 @@
 """TODO: write"""
 from __future__ import annotations
-from importlib import resources
 from typing import TYPE_CHECKING
+from importlib import resources
 
+from . import defaults
+from copy import copy
 from pathlib import Path
 import pytomlpp as toml
 from xdg import XDG_DATA_HOME, XDG_CONFIG_HOME
 
-from . import defaults
-from .base import NAMES, constant, LinkMode
-from .error import (
-    ProjectDataMissingError,
-    TemplateDataMissingError,
+from .base import (
+    NAMES,
+    constant,
+    LinkMode,
+    ModCommand,
+    RemoveCommand,
+    AddCommand,
+    UpdateCommand,
 )
 
 if TYPE_CHECKING:
-    from typing import Optional, List, Dict
+    from typing import Dict, Final, List, Optional
 
 
-def toml_load(path_obj: Path, missing_ok: bool = False) -> Dict:
-    """TODO: write"""
-    # add branch to check if the file exists / fails loading
-    try:
-        return toml.loads(path_obj.read_text())
-    except FileNotFoundError as err:
-        if missing_ok:
-            return {}
-        else:
-            raise err from None
+class TOMLLoader:
+    @staticmethod
+    def load(source: Path, missing_ok: bool = False) -> Dict:
+        try:
+            return toml.loads(source.read_text())
+        except FileNotFoundError as err:
+            if missing_ok:
+                return {}
+            else:
+                raise err from None
 
+    @staticmethod
+    def default_template():
+        return toml.loads(resources.read_text(defaults, "template.toml"))
 
-def toml_dump(path_obj: Path, dct: Dict) -> None:
-    """TODO: write"""
-    path_obj.write_text(toml.dumps(dct))
+    @staticmethod
+    def default_config():
+        return toml.loads(resources.read_text(defaults, "config.toml"))
 
 
 def _merge_iter(*dcts: Dict):
@@ -55,6 +63,36 @@ def _merge(*dcts: Dict) -> Dict:
     return {k: v for k, v in _merge_iter(*dcts)}
 
 
+class TemplateDict(dict):
+    def __init__(self, source: Path):
+        return super().__init__(
+            _merge(TOMLLoader.default_template(), TOMLLoader.load(source))
+        )
+
+    def dump(self, target: Path) -> None:
+        target.write_text(toml.dumps(self))
+
+    def apply_modification(self, mod: ModCommand):
+        match mod:
+            case RemoveCommand(mode, source):
+                self[NAMES.convert_mode(mode)].remove(source)
+
+            case AddCommand(mode, source, index):
+                self[NAMES.convert_mode(mode)].insert(index, source)
+
+            case UpdateCommand(mode, source, target):
+                self[NAMES.convert_mode(mode)] = [
+                    target if val == source else val
+                    for val in self[NAMES.convert_mode(mode)]
+                ]
+
+
+class NamedTemplateDict(TemplateDict):
+    def __init__(self, name: str) -> None:
+        self.name = name
+        super().__init__(template_linker.file_path(name) / NAMES.template_toml)
+
+
 class Config:
     """TODO: write"""
 
@@ -62,9 +100,9 @@ class Config:
         """TODO: write"""
         self.working_dir = working_dir
         self._dct = _merge(
-            DefaultLoader.config(),
-            toml_load(self.global_path, missing_ok=True),
-            toml_load(self.local_path, missing_ok=True),
+            TOMLLoader.default_config(),
+            TOMLLoader.load(self.global_path, missing_ok=True),
+            TOMLLoader.load(self.local_path, missing_ok=True),
         )
         self.user = self._dct["user"]
         self.render = self._dct["render"]
@@ -93,11 +131,6 @@ class _DataPath:
         """TODO: write"""
         return XDG_DATA_HOME / "texproject"
 
-    @constant
-    def resource_dir(self) -> Path:
-        """TODO: write"""
-        return self.data_dir
-
     @constant
     def template_dir(self) -> Path:
         """TODO: write"""
@@ -152,6 +185,10 @@ class _JinjaTemplatePath:
         return "arxiv_autotex.txt"
 
 
+JINJA_PATH: Final = _JinjaTemplatePath()
+DATA_PATH: Final = _DataPath()
+
+
 def relative(base: str):
     """TODO: write"""
 
@@ -261,24 +298,20 @@ class ProjectPath:
             )
 
 
-JINJA_PATH = _JinjaTemplatePath()
-DATA_PATH = _DataPath()
-
-
 class _BaseLinker:
     """TODO: write"""
 
-    def __init__(self, dir_path: Path, suffix: str, user_str: str):
+    def __init__(self, dir_path: Path, suffix: str, user_str: str) -> None:
         """TODO: write"""
-        self.user_str = user_str
-        self.dir_path = dir_path
-        self.suffix = suffix
+        self.user_str: Final = user_str
+        self.dir_path: Final = dir_path
+        self.suffix: Final = suffix
 
     def valid_path(self, path: Path) -> bool:
         """TODO: write"""
         return path.suffix == self.suffix
 
-    def list_names(self) -> List:
+    def list_names(self) -> List[str]:
         """TODO: write"""
         return sorted(
             [path.stem for path in self.dir_path.iterdir() if self.valid_path(path)]
@@ -292,52 +325,20 @@ class _BaseLinker:
 class _FileLinker(_BaseLinker):
     """TODO: write"""
 
-    def __init__(self, suffix: str, user_str: str, mode: LinkMode):
+    def __init__(self, suffix: str, user_str: str, mode: LinkMode) -> None:
         """TODO: write"""
         super().__init__(
-            DATA_PATH.resource_dir / NAMES.resource_subdir(mode), suffix, user_str
+            DATA_PATH.data_dir / NAMES.resource_subdir(mode), suffix, user_str
         )
-        self.mode = mode  # type: LinkMode
-
-
-class DefaultLoader:
-    @staticmethod
-    def template() -> Dict:
-        return toml.loads(resources.read_text(defaults, "template.toml"))
-
-    @staticmethod
-    def config() -> Dict:
-        return toml.loads(resources.read_text(defaults, "config.toml"))
-
-
-def toml_load_local_template(path: Path) -> Dict:
-    """TODO: write"""
-    try:
-        template = toml_load(path)
-    except FileNotFoundError as err:
-        raise ProjectDataMissingError(
-            path, message="The local template file is missing."
-        ) from err
-    return _merge(DefaultLoader.template(), template)
-
-
-def toml_load_system_template(path: Path, user_str: str, name: Optional[str] = None):
-    """TODO: write"""
-    try:
-        template = toml_load(path)
-    except FileNotFoundError as err:
-        raise TemplateDataMissingError(path, user_str=user_str, name=name) from err
-    return _merge(DefaultLoader.template(), template)
+        self.mode = mode
 
 
 class _TemplateLinker(_BaseLinker):
     """TODO: write"""
 
-    def load_template(self, name: str) -> Dict:
+    def load_template(self, name: str) -> TemplateDict:
         """TODO: write"""
-        return toml_load_system_template(
-            self.file_path(name) / NAMES.template_toml, self.user_str, name=name
-        )
+        return TemplateDict.from_path(self.file_path(name) / NAMES.template_toml)
 
     def valid_path(self, path: Path):
         """TODO: write"""
@@ -348,12 +349,12 @@ class _TemplateLinker(_BaseLinker):
         )
 
 
-macro_linker = _FileLinker(".sty", "macro file", LinkMode.macro)
-citation_linker = _FileLinker(".bib", "citation file", LinkMode.citation)
-style_linker = _FileLinker(".sty", "style file", LinkMode.style)
-template_linker = _TemplateLinker(DATA_PATH.template_dir, "", "template")
+macro_linker: Final = _FileLinker(".sty", "macro file", LinkMode.macro)
+citation_linker: Final = _FileLinker(".bib", "citation file", LinkMode.citation)
+style_linker: Final = _FileLinker(".sty", "style file", LinkMode.style)
+template_linker: Final = _TemplateLinker(DATA_PATH.template_dir, "", "template")
 
-LINKER_MAP = {
+LINKER_MAP: Final = {
     LinkMode.macro: macro_linker,
     LinkMode.citation: citation_linker,
     LinkMode.style: style_linker,
