@@ -10,8 +10,8 @@ from . import __repo__
 
 if TYPE_CHECKING:
     from pathlib import Path
-    from typing import Iterable
-    from .filesystem import _FileLinker
+    from typing import Iterable, Final
+    from .template import _FileLinker
 
 REPO_FORMATTED = click.style(__repo__, fg="bright_blue")
 
@@ -112,4 +112,4 @@ class _MessageFormatter:
         return self._apply_style(message, "err", "err")
 
 
-FORMAT_MESSAGE = _MessageFormatter()
+FORMAT_MESSAGE: Final = _MessageFormatter()
